import redis
from redis.exceptions import RedisError
class LuaScriptError(RedisError):
    pass


class LuaCall(object):
    """
    A class that helps you treat lua functions like they were actual functions.
    Example:
    >>> lua_str = '''
                    local val = ARGV[1]*ARGV[2]
                    redis.call('set', KEYS[1], val)
                    return redis.call('get', KEYS[1])
                '''
    >>> mult = LuaCall(lua_str, myConnection) # also acceptable: mult = LuaCall(open('mult.lua'), myConnection)
    >>> print mult(keys = ('foo',), args = (3,10))
    30

    """
    def __init__(self, sourceOrFile, redisConn = None):
        """
        construct the functin
        @param sourceOrFile either a lua string, or a reference to a file in read mode containing the source
        @param redisConn a redis connection. if not given here, you'll have to give it on each call (useful for master/slave)
        """

        self.source = sourceOrFile if type(sourceOrFile) == str else sourceOrFile.read()
        self.conn = redisConn

        #if a connection was given - try to preload the function. if not - it will have to be given later
        if self.conn:
            self.__load()

    def __call__(self,  keys=(), args=(), conn = None):

        conn = conn or self.conn
        #try to execute
        try:
            return conn.execute_command('EVALSHA', self.sha, len(keys), *(keys + args))
        except RedisError, e:
            #check for script doesn't exist error
            if e.message.startswith('NOSCRIPT'):

                #try and reload
                self.__load(conn)

                #one more time, with feeling!
                try:

                    return conn.execute_command('EVLASHA', self.sha, len(keys), *(keys + args))
                except redis.RedisError, e:
                    raise LuaScriptError("Could not execute lua call: %s" % e.message)
            else:
                raise LuaScriptError(e)


    def __load(self, conn = None):
        """
        Silently preload the function to redis to be used in the future
        """
        conn = conn or self.conn
        self.sha = conn.execute_command('SCRIPT', 'LOAD', self.source)

    def isCached(self, conn = None):
        """
        Check if our function exists
        """
        conn = conn or self.conn
        return conn.execute_command('SCRIPT', 'EXISTS', self.sha)








from distutils.core import setup
setup(name='kickass_redis',
    version='0.1.4',
    package_dir={'kickass_redis': 'src'},
    packages=['kickass_redis', 'kickass_redis.patterns', 'kickass_redis.patterns.object_store'],
    requires=['redis'],
    url='https://github.com/EverythingMe/kickass-redis',
    author='Dvir Volk',
    description= 'A loose framework of kick-ass redis patterns',
    long_description="""================================================================
Kickass-Redis - a loose framework of Redis based data solutions
================================================================

This project aims to create a repository of useful python libraries built on top of redis (and using each other), to automate data modeling with Redis.

Redis is relatively low level, and while it is simple to start using, getting a good knowledge of how to model problems with it in an efficient way can be tricky. So I've created this project to wrap common use cases, into a loose framework of redis based solutions for real world problems.

Components:
==================

1. Object Store
~~~~~~~~~~~~~~~~

a fast yet simple ORM (well, OM actually) that automates creation, indexing and searching for complex objects using redis.

Indexes include: simple string index, numeric index that supports sorting and ranges, simplistic full text index, and a unique key.

2. Bitmap Counter
~~~~~~~~~~~~~~~~~

efficient unique value counter (to be used mostly as a unique users counter) with time slots, making use of redis bitmaps.

It makes use of new redis-2.6 commands BITCOUNT and BITOP, so it will not function on redis-2.4.

3. LuaCall
~~~~~~~~~~~~~~~~~

A convenience wrapper that allows you to edit, precache and call Lua scripts available in redis-2.6, as if they were native python functions.

4. Id Generator
~~~~~~~~~~~~~~~~~~

Used in the object store, this can also be used standalone, as a centralized unique, incremental id generator using redis. To optimize performance, it reserves in local memory many ids when accessing redis, which can be tuned.

5. Redis Unit
~~~~~~~~~~~~~~~~~

A unit-test like set of assertions about redis data to be used to validate the data inside a redis database.

"""
)
